package com.caimei365.commodity.controller;

import com.caimei365.commodity.annotation.Idempotent;
import com.caimei365.commodity.annotation.IpSave;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.dto.CollarCouponsDto;
import com.caimei365.commodity.model.dto.RedeemCouponsDto;
import com.caimei365.commodity.model.vo.CouponVo;
import com.caimei365.commodity.service.CouponService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.util.Map;

/**
 * 优惠券相关
 *
 * @author : plf
 * @date : 2021/8/12
 */
@Api(tags = "优惠券相关API")
@RestController
@RequestMapping("/commodity/coupon")
public class CouponApi {
    private CouponService couponService;

    @Autowired
    public void setCouponService(CouponService couponService) {
        this.couponService = couponService;
    }

    @ApiOperation("已领取优惠券中心")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = false, name = "status", value = "使用状态: 1未使用 2已使用 3已失效"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/center")
    @IpSave(saveName = "领券中心",saveParams = true)
    public ResponseJson<PageInfo<CouponVo>> couponCenter(Integer userId, Integer status,
                                                         @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                         @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.couponCenter(userId, status, pageNum, pageSize);
    }

    @ApiOperation("活动页")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "couponId", value = "优惠券id"),
            @ApiImplicitParam(required = true, name = "source", value = "来源 : 1 小程序 ; 2 网站"),
            @ApiImplicitParam(required = true, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/activity/page")
    public ResponseJson<Map<String, Object>> activityPage(Integer couponId, Integer userId, Integer source,
                                                          @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                          @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (couponId == null || source == null || userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.activityPage(couponId, userId, source, pageNum, pageSize);
    }

    @ApiOperation("领券中心列表")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "机构用户id,0不传"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/collar/list")
    public ResponseJson<PageInfo<CouponVo>> collarCouponsList(Integer userId,
                                                              @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                              @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return couponService.collarCouponsList(userId, pageNum, pageSize);
    }

    @ApiOperation("领券中心列表备用接口")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "机构用户id,0不传"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/center/list")
    public ResponseJson<PageInfo<CouponVo>> couponCenter(Integer userId,
                                                         @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                         @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return couponService.collarCenterList(userId, pageNum, pageSize);
    }

    @ApiOperation("领取优惠券")
    @PostMapping("/collar")
    @IpSave(saveName = "领取优惠券",saveParams = true)
    @Idempotent(prefix = "idempotent_collarCoupons", keys = {"#couponsDto"}, expire = 5)
    public ResponseJson<String> collarCoupons(CollarCouponsDto couponsDto) throws ParseException {
        if (couponsDto.getCouponId() == null || couponsDto.getSource() == null || couponsDto.getUserId() == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.collarCoupons(couponsDto);
    }

    @ApiOperation("兑换优惠券")
    @PostMapping("/redeem")
    public ResponseJson<CouponVo> redeemCoupons(RedeemCouponsDto redeemCouponsDto) {
        if (redeemCouponsDto.getUserId() == null || redeemCouponsDto.getSource() == null) {
            return ResponseJson.error("参数异常", null);
        }
        if (StringUtils.isBlank(redeemCouponsDto.getRedemptionCode())) {
            return ResponseJson.error("请输入兑换码", null);
        }
        return couponService.redeemCoupons(redeemCouponsDto);
    }

    @ApiOperation("统计已领取优惠券数量")
    @ApiImplicitParam(required = true, name = "userId", value = "机构用户id")
    @GetMapping("/coupons/count")
    public ResponseJson<Map<String, Integer>> couponsCount(Integer userId) {
        if (userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.couponsCount(userId);
    }

    @ApiOperation("商品详情相关优惠券")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = true, name = "source", value = "来源 : 1 小程序 ; 2 网站"),
            @ApiImplicitParam(required = true, name = "productId", value = "商品id"),
            @ApiImplicitParam(required = true, name = "status", value = "状态: 1未领取 2已领取")
    })
    @GetMapping("/details/coupons")
    public ResponseJson<Map<String, Object>> detailsCoupons(Integer userId, Integer productId, Integer source, Integer status) {
        if (productId == null || source == null || status == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.detailsCoupons(userId, productId, source, status);
    }

    @ApiOperation("协销价值优惠券列表")
    @GetMapping("/money/coupon/list")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    public ResponseJson<PageInfo<CouponVo>> moneyCouponList(Integer userId,
                                                            @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                            @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (null == userId) {
            return ResponseJson.error("用户Id不为空！", null);
        }
        return couponService.findMoneyCoupons(userId, pageNum, pageSize);
    }

    @ApiOperation("根据优惠券ID查优惠券信息")
    @GetMapping("/coupon/detail")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "couponId", value = "优惠券Id")
    })
    public ResponseJson<CouponVo> findCouponDetail(Integer couponId) {
        return couponService.findCouponDetail(couponId);
    }
}
