package com.caimei365.commodity.controller;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.commodity.annotation.IpSave;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.po.SearchHistoryPo;
import com.caimei365.commodity.model.vo.SearchFrequencyVo;
import com.caimei365.commodity.model.vo.SearchKeywordyVo;
import com.caimei365.commodity.service.SearchProductService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 搜索商品（阿里云搜索）
 *
 * @author : Charles
 * @date : 2021/4/7
 */
@Api(tags = "搜索商品API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity/search/query")
public class SearchProductApi {

    private final SearchProductService searchProductService;


    @ApiOperation("根据keyword或分类搜品牌")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
            @ApiImplicitParam(required = false, name = "id", value = "bigTypeID/smallTypeID/tinyTypeID"),
            @ApiImplicitParam(required = false, name = "newFlag", value = "查询新品标记，默认0，新品1"),
            @ApiImplicitParam(required = false, name = "promotionFlag", value = "查询促销标记，默认0，促销1"),
            @ApiImplicitParam(required = false, name = "idType", value = "id类型：1一级分类，2二级分类，3三级分类"),
            @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构，5医美机构")
    })
    @GetMapping("/brand")
    public ResponseJson<Set<HashMap<String, Object>>> queryBrand(String keyword, Integer id, Integer idType,
                                                                 @RequestParam(value = "newFlag", defaultValue = "0") Integer newFlag,
                                                                 @RequestParam(value = "promotionFlag", defaultValue = "0") Integer promotionFlag,
                                                                 @RequestParam(value = "identity", defaultValue = "0") Integer identity) {
        return searchProductService.queryBrand(keyword, id, idType, newFlag, promotionFlag, identity);
    }

    /**
     * 根据关键词搜索商品
     * <p>
     * spi旧接口：/search/query/product
     *
     * @param keyword 搜索关键字
     * @return JsonStr(list)
     */
    @ApiOperation("根据keyword搜索商品")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
            @ApiImplicitParam(required = false, name = "brandIds", value = "品牌id字符串，逗号隔开"),
            @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构，5医美机构"),
            @ApiImplicitParam(required = false, name = "newFlag", value = "查询新品标记，默认0，新品1"),
            @ApiImplicitParam(required = false, name = "promotionFlag", value = "查询促销标记，默认0，促销1"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
            @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @IpSave(saveName = "搜索商品", saveParams = true)
    @GetMapping("/product")
    public ResponseJson<String> queryProductByKeyword(String keyword, String brandIds,
                                                      @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                                      @RequestParam(value = "newFlag", defaultValue = "0") Integer newFlag,
                                                      @RequestParam(value = "promotionFlag", defaultValue = "0") Integer promotionFlag,
                                                      @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                      @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                                      @RequestParam(value = "productFlag", defaultValue = "1") int productFlag,
                                                      @RequestParam(value = "linkageFlag", defaultValue = "0") int linkageFlag,
                                                      String sortField, Integer sortType) {
        return searchProductService.queryProductByKeyword(keyword, brandIds, identity, newFlag, promotionFlag, pageNum, pageSize, sortField, sortType);
    }

    /**
     * 根据分类Id搜索商品
     * <p>
     * spi旧接口：/search/query/product/type
     *
     * @param id     bigTypeID/smallTypeID/tinyTypeID
     * @param idType id类型：1一级分类，2二级分类，3三级分类
     */
    @ApiOperation("根据分类Id搜索商品")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "id", value = "bigTypeID/smallTypeID/tinyTypeID"),
            @ApiImplicitParam(required = true, name = "idType", value = "id类型：1一级分类，2二级分类，3三级分类"),
            @ApiImplicitParam(required = false, name = "brandIds", value = "品牌id字符串，逗号隔开"),
            @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构，5医美机构"),
            @ApiImplicitParam(required = false, name = "newFlag", value = "查询新品标记，默认0，新品1"),
            @ApiImplicitParam(required = false, name = "promotionFlag", value = "查询促销标记，默认0，促销1"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
            @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @IpSave(saveName = "点击分类搜索商品", saveParams = true)
    @GetMapping("/product/type")
    public ResponseJson<String> queryProductByType(Integer id, Integer idType, String brandIds,
                                                   @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                                   @RequestParam(value = "newFlag", defaultValue = "0") Integer newFlag,
                                                   @RequestParam(value = "promotionFlag", defaultValue = "0") Integer promotionFlag,
                                                   @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                   @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                                   String sortField, Integer sortType) {
        return searchProductService.queryProductByType(id, idType, brandIds, identity, newFlag, promotionFlag, pageNum, pageSize, sortField, sortType);
    }

    /**
     * 根据小程序运营分类Id搜索商品
     * <p>
     * spi旧接口：/search/query/product/classify
     *
     * @param id       运营级分类Id
     * @param identity 用户身份 1:协销， 2:会员机构
     */
    @ApiOperation("根据小程序运营分类Id搜索商品")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "id", value = "运营级分类Id"),
            @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构，5医美机构"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
            @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @GetMapping("/product/classify")
    public ResponseJson<String> queryProductByClassify(Integer id,
                                                       @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                                       @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                       @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                                       String sortField, Integer sortType) {
        return searchProductService.queryProductByClassify(id, identity, pageNum, pageSize, sortField, sortType);
    }

    /**
     * 根据供应商Id搜索商品（商铺内搜索）
     * <p>
     * spi旧接口：/search/query/product/supplier
     *
     * @param id      供应商Id
     * @param keyword 搜索关键字
     */
    @ApiOperation("根据供应商Id搜索商品（商铺内搜索）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "id", value = "供应商Id"),
            @ApiImplicitParam(required = false, name = "keyword", value = "搜索关键字"),
            @ApiImplicitParam(required = false, name = "identity", value = "用户身份: 0个人，1协销，2会员机构，3供应商，4普通机构，5医美机构"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量"),
            @ApiImplicitParam(required = false, name = "sortField", value = "排序字段：价格price，销量sales，人气favorite"),
            @ApiImplicitParam(required = false, name = "sortType", value = "排序规则：1降序，其他升序")
    })
    @IpSave(saveName = "供应商首页搜索", saveParams = true)
    @GetMapping("/product/shop")
    public ResponseJson<String> querySupplierProduct(Integer id, String keyword,
                                                     @RequestParam(value = "identity", defaultValue = "0") Integer identity,
                                                     @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                     @RequestParam(value = "pageSize", defaultValue = "20") int pageSize,
                                                     String sortField, Integer sortType) {
        return searchProductService.querySupplierProduct(id, keyword, identity, pageNum, pageSize, sortField, sortType);
    }

    /**
     * 根据用户ID查找历史记录
     */
    @ApiOperation("查找搜索历史记录（旧：/product/searchHistory）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/history")
    public ResponseJson<List<SearchHistoryPo>> getSearchHistory(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return searchProductService.getSearchHistory(userId);
    }

    /**
     * 保存搜索历史记录
     */
    @ApiOperation("保存搜索历史记录（旧：/product/history/add）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "用户Id"),
            @ApiImplicitParam(required = true, name = "keyword", value = "搜索关键字")
    })
    @GetMapping("/history/add")
    public ResponseJson<Void> addProductSearchHistory(Integer userId, String keyword) {
        if (null == userId || StringUtils.isEmpty(keyword)) {
            return ResponseJson.error("用户Id或关键字不能为空！", null);
        }
        return searchProductService.addSearchHistory(userId, keyword);
    }

    /**
     * 根据用户ID删除历史记录
     */
    @ApiOperation("删除搜索历史记录（旧：/product/searchHistory/delete ）")
    @ApiImplicitParam(required = true, name = "userId", value = "用户Id")
    @GetMapping("/history/delete")
    public ResponseJson<Void> deleteSearchHistory(Integer userId) {
        if (null == userId) {
            return ResponseJson.error("用户Id不能为空！", null);
        }
        return searchProductService.deleteSearchHistory(userId);
    }

    /**
     * 产品仪器热搜词
     */
    @GetMapping("/hot/keyword")
    public ResponseJson<Map<String, Object>> getHotSearchTerms() {
        return searchProductService.getHotSearchTerms();
    }

    /**
     * 统计关键词及次数
     *
     * @param fromType
     * @param path
     * @param keyword
     * @return
     */
    @GetMapping("/con/keyword")
    public ResponseJson conKeyword(Integer fromType, String path, String keyword) {
        return ResponseJson.success(searchProductService.conKeyword(fromType, path, keyword));
    }

    /**
     * 获取已加入关键词库的关键词进行搜索联动
     * 从redis获取，获取不到则去数据库获取
     *
     * @return
     */
    @PostMapping("/get/keyword/list")
    public ResponseJson<List<SearchKeywordyVo>> getKeywordList(SearchFrequencyVo searchKeywordyVo) {
        if (StringUtils.isEmpty(searchKeywordyVo.getKeyword())) {
            return ResponseJson.error("请输入关键词");
        }
        return ResponseJson.success(searchProductService.getKeywordList(searchKeywordyVo.getKeyword()));
    }

    @GetMapping("/keyword/list")
    public ResponseJson<List<SearchKeywordyVo>> getKeywordLIstByData(String keyword) {
        return ResponseJson.success(searchProductService.getKeywordLIstByData(keyword));
    }

    @GetMapping("/conKeyWord")
    public void conKeyWord() {
        searchProductService.conKeyWord();
    }

}
