package com.caimei365.commodity.controller;

import com.caimei365.commodity.annotation.Idempotent;
import com.caimei365.commodity.annotation.IpSave;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.dto.SecondDto;
import com.caimei365.commodity.model.vo.BrandVo;
import com.caimei365.commodity.model.vo.PaginationVo;
import com.caimei365.commodity.model.vo.SecondDetailVo;
import com.caimei365.commodity.model.vo.SecondListVo;
import com.caimei365.commodity.service.SecondHandService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/14
 */
@Api(tags = "二手商品API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/commodity/second")
public class SecondHandApi {

    private final SecondHandService secondHandService;

    /**
     * 二手商品列表
     */
    @ApiOperation("二手商品列表（旧：/product/getSecondHandProductList）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = false, name = "secondHandType", value = "二手商品分类，1二手仪器，2临期产品，3其他"),
            @ApiImplicitParam(required = false, name = "instrumentType", value = "二手仪器分类的类型，1轻光电、2重光电、3耗材配件"),
            @ApiImplicitParam(required = false, name = "name", value = "二手商品名称搜索关键词"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @IpSave(saveName = "二手商品列表",saveParams = true)
    @GetMapping("/list")
    public ResponseJson<PaginationVo<SecondListVo>> getSecondHandList(Integer userId, Integer secondHandType, Integer instrumentType, String name,
                                                                      @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                      @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return secondHandService.getSecondHandList(userId, secondHandType, instrumentType, name, pageNum, pageSize);
    }

    /**
     * 获取二手商品详情
     *
     * @param productId 商品表的商品Id
     */
    @ApiOperation("二手商品详情（旧：/product/getSecondHandProductDetail）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = true, name = "productId", value = "商品Id")
    })
    @IpSave(saveName = "获取二手商品详情",saveParams = true)
    @GetMapping("/detail")
    public ResponseJson<SecondDetailVo> getSecondDetail(Integer userId, Integer productId) {
        return secondHandService.getSecondHandDetail(userId, productId);
    }

    /**
     * 发布二手商品
     *
     * @param secondDto 对象内容描述{
     *                  "secondHandType" : 二手商品分类，1二手仪器，2临期产品，3其他
     *                  ,"instrumentType" : 二手仪器分类的类型，1轻光电、2重光电、3耗材配件(仅适用于二手仪器分类多个用英文逗号分分隔)
     *                  ,"brandId" : 品牌Id
     *                  ,"name" : 商品名称
     *                  ,"fixedYears" : 出厂日期格式：2020年6月
     *                  ,"maturityYears" : 产品到期日格式：2020年6月（仅适用于临期产品）
     *                  ,"companyName" : 公司名称
     *                  ,"price" : 交易价格
     *                  ,"detailTalkFlag" : 是否启用详聊，1不开启，2开启(开启详聊不展示交易价)
     *                  ,"normalPrice" : 市场价
     *                  ,"originalPrice" : 采购价/原价(该二手原始购买价格）
     *                  ,"stock" : 数量（等同库存）
     *                  ,"productQuality" : 商品成色
     *                  ,"contactName" : 联系人名字
     *                  ,"contactMobile" : 联系方式
     *                  ,"secondProductType" : 设备类型：1医美、2非医美
     *                  ,"townId" : 县区ID
     *                  ,"address" : 详细地址
     *                  ,"image" : 图片信息
     *                  ,"productDetails" : 商品详细信息
     *                  ,"source" : 信息来源 1网站  2CRM  3后台
     *                  ,"publishIdentity" :  发布者身份  1个人身份，2公司身份
     *                  ,"licenseImage" : 营业执照照片
     *                  ,"fileType" : 商品包含的文件类型，1，仪器原始合同，2，仪器注册证，3，仪器发票，4，仪器报关单，5，无任何文件'
     *                  (fileType字符串逗号拼接)
     *                  ,"checkCode" : 验证码
     *                  ,"authenticationBackImage" : 身份证背面照片
     *                  ,"cardNumber" : 收款卡号
     *                  }
     * @param headers   HttpHeaders
     */
    @ApiOperation("发布二手商品（旧：/product/releaseSecondHandProduct）")
    @IpSave(saveName = "发布二手商品",saveParams = false)
    @Idempotent(prefix = "idempotent_secondhand", keys = {"#secondDto"}, expire = 5)
    @PostMapping("/release")
    public ResponseJson releaseSecondHand(SecondDto secondDto, @RequestHeader HttpHeaders headers) {
        return secondHandService.releaseSecondHand(secondDto, headers);
    }

    /**
     * 二手下单页面商品
     *
     * @param searchKeyword  搜索关键词
     * @param secondHandType 二手商品分类，1二手仪器，2临期产品，3其他
     * @param instrumentType 二手仪器分类的类型，1轻光电、2重光电、3耗材配件
     */
    @ApiOperation("二手下单页面商品（旧：/product/getOrderSecondHandProductList）")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "searchKeyword", value = "二手商品搜索关键词"),
            @ApiImplicitParam(required = false, name = "secondHandType", value = "二手商品分类，1二手仪器，2临期产品，3其他"),
            @ApiImplicitParam(required = false, name = "instrumentType", value = "二手仪器分类的类型，1轻光电、2重光电、3耗材配件"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/order/products")
    public ResponseJson<PaginationVo<SecondListVo>> getOrderPageSecondList(String searchKeyword, Integer secondHandType, Integer instrumentType,
                                                                           @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                                           @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return secondHandService.getOrderPageSecondList(searchKeyword, secondHandType, instrumentType, pageNum, pageSize);
    }

    /**
     * 二手发布页-品牌列表
     */
    @ApiOperation("二手发布页-品牌列表(旧：/product/brandList)")
    @GetMapping("/brands")
    public ResponseJson<List<BrandVo>> getBrandList() {
        return secondHandService.getBrandList();
    }


    /**
     * 计算二手商品浏览量
     */
    @ApiOperation("计算二手商品浏览量(旧：/product/updateSecondHandProductCount)")
    @ApiImplicitParam(required = true, name = "productId", value = "商品Id")
    @GetMapping("/views")
    public ResponseJson<Integer> updateSecondHandViews(Integer productId){
        if (null == productId) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        return secondHandService.updateSecondHandViews(productId);
    }

    /**
     * 二手商品-相关推荐
     *
     * @param productId 商品Id
     */
    @ApiOperation("二手商品-相关推荐(旧：/product/getSecondHandProductRecommend)")
    @ApiImplicitParam(required = false, name = "productId", value = "商品Id")
    @GetMapping("/recommend")
    public ResponseJson<List<SecondListVo>> getSecondRecommends(Integer productId) {
        if (null == productId) {
            return ResponseJson.error("商品Id不能为空！", null);
        }
        return secondHandService.getSecondRecommends(productId);
    }
}
