package com.caimei365.commodity.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.caimei365.commodity.components.PriceUtilService;
import com.caimei365.commodity.feign.ToolsFeign;
import com.caimei365.commodity.mapper.*;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.dto.CollarCouponsDto;
import com.caimei365.commodity.model.dto.RedeemCouponsDto;
import com.caimei365.commodity.model.po.CouponClubPo;
import com.caimei365.commodity.model.po.CouponRedemptionCodePo;
import com.caimei365.commodity.model.vo.*;
import com.caimei365.commodity.service.CouponService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.SneakyThrows;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/8/12
 */
@Service
public class CouponServiceImpl implements CouponService {
    @Value("${spring.cloud.config.profile}")
    private String profile;
    @Resource
    private ToolsFeign toolsFeign;
    @Resource
    private CouponMapper couponMapper;
    @Resource
    private ShopMapper shopMapper;
    @Resource
    private PageMapper pageMapper;
    @Resource
    private PriceMapper priceMapper;
    @Resource
    private PriceUtilService priceUtilService;
    @Resource
    private MessageCenterMapper messageCenterMapper;

    /**
     * 已领取优惠券中心
     *
     * @param userId   机构用户id
     * @param status   使用状态: 1未使用 2已使用 3已失效
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    @Override
    public ResponseJson<PageInfo<CouponVo>> couponCenter(Integer userId, Integer status, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CouponVo> couponList = couponMapper.findCouponClub(userId, status);
        couponList.forEach(coupon -> {
            setShopName(coupon);
            coupon.setUseStatus(status);
        });
        PageInfo<CouponVo> pageInfo = new PageInfo<>(couponList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * set店铺名称
     *
     * @param coupon 优惠券
     */
    private void setShopName(CouponVo coupon) {
        if (coupon.getCouponType() == 3 && coupon.getShopId() != null) {
            //店铺券
            ShopVo shop = shopMapper.getProductShopById(coupon.getShopId());
            if (shop != null && StringUtils.isNotBlank(shop.getName())) {
                String shopName = shop.getName();
                if (shopName.length() > 7) {
                    shopName = shopName.substring(0, 6) + "...";
                }
                coupon.setShopName(shopName);
            }
        }
    }

    /**
     * 活动券部分商品活动数据
     *
     * @param couponId 活动券id
     * @param userId   机构用户id
     * @param source   来源 : 1 小程序 ; 2 网站
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    @Override
    public ResponseJson<Map<String, Object>> activityPage(Integer couponId, Integer userId, Integer source, int pageNum, int pageSize) {
        Map<String, Object> map = new HashMap<>(2);
        ActivityCouponVo coupon = couponMapper.findCouponById(couponId);
        if (coupon == null) {
            return ResponseJson.error("活动已失效", null);
        }
        // 用户身份：0个人，1协销，2会员机构，3供应商，4普通机构
        Integer identity = priceMapper.getIdentityByUserId(userId);
        // 会员机构类型：1医美，2生美
        if (null != identity && identity == 2) {
            Integer clubType = priceMapper.getClubTypeById(userId);
            if (null != clubType && clubType == 1) {
                identity = 5;
            }
        }
        PageHelper.startPage(pageNum, pageSize);
        List<ProductItemVo> productList = couponMapper.findCouponProduct(couponId, source, identity);
        productList.forEach(p -> {
            p.setCouponsLogo(true);
            priceUtilService.setProductDetails(userId, p, 0);
        });
        PageInfo<ProductItemVo> pageInfo = new PageInfo<>(productList);
        map.put("coupon", coupon);
        map.put("pageInfo", pageInfo);
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson<PageInfo<CouponVo>> collarCouponsList(Integer userId, int pageNum, int pageSize) {
        Date registerTime = couponMapper.findUserRegisterTime(userId);
        PageHelper.startPage(pageNum, pageSize);
        List<CouponVo> couponList = couponMapper.findCouponList(userId, registerTime);
        couponList.forEach(this::setShopName);
        PageInfo<CouponVo> pageInfo = new PageInfo<>(couponList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<String> collarCoupons(CollarCouponsDto couponsDto) throws ParseException {
        Integer id = couponMapper.findCouponClubByUserId(couponsDto.getUserId(), couponsDto.getCouponId());
        if (id != null && id > 0) {
            return ResponseJson.error("已经领取过了", null);
        }
        //超级会员券领取限制
        List<Integer> couponIds = couponMapper.findVipCoupon();
        if (couponIds.contains(couponsDto.getCouponId())) {
            Integer vip = couponMapper.findVip(couponsDto.getUserId());
            if (vip == null || vip <= 0) {
                return ResponseJson.error("未成为超级会员无法领取超级会员券！");
            }
        }
        CouponClubPo couponClub = new CouponClubPo();
        BeanUtils.copyProperties(couponsDto, couponClub);
        couponClub.setStatus("1");
        couponClub.setCreateDate(new Date());
        couponClub.setDelFlag("0");
        couponMapper.insertCouponClub(couponClub);
        return ResponseJson.success("领取成功");
    }


    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseJson<CouponVo> redeemCoupons(RedeemCouponsDto redeemCouponsDto) {
        CouponRedemptionCodePo redemptionCode = couponMapper.findCouponRedemptionCode(redeemCouponsDto.getRedemptionCode());
        if (redemptionCode == null) {
            return ResponseJson.error("请输入正确的兑换码", null);
        }
        if (redemptionCode.getStatus() == 2) {
            return ResponseJson.error("兑换码已使用，请更换兑换码进行兑换", null);
        }
        Date registerTime = couponMapper.findUserRegisterTime(redeemCouponsDto.getUserId());
        CouponVo coupon = couponMapper.getCoupons(redemptionCode.getCouponId(), registerTime);
        Date date = new Date();
        // 判断是否在领取期限内
        if (coupon == null || date.compareTo(coupon.getStartDate()) < 0 || date.compareTo(coupon.getEndDate()) > 0) {
            return ResponseJson.error("兑换的优惠券已失效", null);
        }
        if (coupon.getCouponType() == 2 && !redeemCouponsDto.getUserId().equals(coupon.getUserId())) {
            return ResponseJson.error("该优惠券属于其他用户，您不能兑换", null);
        }
        if (coupon.getCouponType() == 4) {
            if (registerTime == null || registerTime.compareTo(coupon.getStartDate()) < 0) {
                return ResponseJson.error("该优惠券只供新用户使用，您不能兑换", null);
            }
        }
        setShopName(coupon);
        CouponClubPo couponClub = new CouponClubPo();
        couponClub.setCouponId(redemptionCode.getCouponId());
        couponClub.setUserId(redeemCouponsDto.getUserId());
        couponClub.setSource(redeemCouponsDto.getSource());
        couponClub.setStatus("1");
        couponClub.setCreateDate(new Date());
        couponClub.setDelFlag("0");
        couponMapper.insertCouponClub(couponClub);
        couponMapper.updateRedemptionCode(redemptionCode.getId(), couponClub.getId());
        // 设置使用有效期
        coupon.setStartDate(new Date());
        coupon.setEndDate(coupon.getUsePeriod());
        return ResponseJson.success(coupon);
    }

    @Override
    public ResponseJson<Map<String, Integer>> couponsCount(Integer userId) {
        Map<String, Integer> map = new HashMap<>(3);
        List<CouponVo> unusedList = couponMapper.findCouponClub(userId, 1);
        List<CouponVo> usedList = couponMapper.findCouponClub(userId, 2);
        List<CouponVo> expiredList = couponMapper.findCouponClub(userId, 3);
        int unusedNum = unusedList == null ? 0 : unusedList.size();
        int usedNum = usedList == null ? 0 : usedList.size();
        int expiredNum = expiredList == null ? 0 : expiredList.size();
        map.put("unusedNum", unusedNum);
        map.put("usedNum", usedNum);
        map.put("expiredNum", expiredNum);
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson<Map<String, Object>> detailsCoupons(Integer userId, Integer productId, Integer source, Integer status) {
        Map<String, Object> map = new HashMap<>(3);
        ProductItemVo product = pageMapper.getProductItemById(productId);
        if (product == null) {
            return ResponseJson.error("商品数据异常", null);
        }
        List<Integer> vipCoupon = couponMapper.findVipCoupon();
        // 注册时间
        Date registerTime = couponMapper.findUserRegisterTime(userId);
        //未领取
        List<CouponVo> notCouponList = couponMapper.findCouponList(userId, registerTime);
        filterCoupon(source, product, notCouponList);
        notCouponList.forEach(n -> n.setCouponBtnType(0));
        //超级会员优惠券移除
        if (notCouponList != null && notCouponList.size() > 0) {
            notCouponList.removeIf(couponVo -> vipCoupon.contains(couponVo.getCouponId()));
        }
        map.put("notCouponNum", notCouponList == null ? 0 : notCouponList.size());
        //已领取
        List<CouponVo> couponList = couponMapper.findCouponClub(userId, 1);
        couponList.forEach(c -> c.setCouponBtnType(1));
        //超级会员优惠券移除
        if (couponList != null && couponList.size() > 0) {
            couponList.removeIf(couponVo -> vipCoupon.contains(couponVo.getCouponId()));
        }
        filterCoupon(source, product, couponList);
        map.put("couponNum", couponList == null ? 0 : couponList.size());
        if (status == 1) {
            //未领取
            map.put("couponList", notCouponList);
        } else {
            //已领取
            map.put("couponList", couponList);
        }
        List<CouponVo> list = new ArrayList<>();
        if (notCouponList != null) {
            list.addAll(notCouponList);
        }
        if (couponList != null) {
            list.addAll(couponList);
        }
        list.sort((o1, o2) -> o2.getCouponAmount().compareTo(o1.getCouponAmount()));
        map.put("list", list);
        return ResponseJson.success(map);
    }

    /**
     * 用户可查看的价值优惠券
     *
     * @param userId
     * @param pageNum
     * @param pageSize
     * @return
     */
    @Override
    public ResponseJson<PageInfo<CouponVo>> findMoneyCoupons(Integer userId, int pageNum, int pageSize) {
        Date registerTime = couponMapper.findUserRegisterTime(userId);
        PageHelper.startPage(pageNum, pageSize);
        List<CouponVo> cop = couponMapper.findMoneyCoupons(userId, registerTime);
        PageInfo<CouponVo> pageInfo = new PageInfo<>(cop);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<CouponVo> findCouponDetail(Integer couponId) {
        CouponVo couponVo = couponMapper.findCouponDetail(couponId);
        return ResponseJson.success(couponVo);
    }

    @Override
    public ResponseJson<PageInfo<CouponVo>> collarCenterList(Integer userId, int pageNum, int pageSize) {
        Date registerTime = couponMapper.findUserRegisterTime(userId);
        Integer permission = couponMapper.findUserPermission(userId);
        //如果是协销，userid置为0查所有
        if (1 == permission) {
            userId = 0;
        }
        PageHelper.startPage(pageNum, pageSize);
        List<CouponVo> couponList = couponMapper.findCouponList(userId, registerTime);
        couponList.forEach(this::setShopName);
        PageInfo<CouponVo> pageInfo = new PageInfo<>(couponList);
        return ResponseJson.success(pageInfo);
    }

    /**
     * 过滤与商品无关的优惠券
     *
     * @param source
     * @param product
     * @param couponList
     */
    private void filterCoupon(Integer source, ProductItemVo product, List<CouponVo> couponList) {
        if (couponList != null && couponList.size() > 0) {
            Iterator<CouponVo> iterator = couponList.iterator();
            while (iterator.hasNext()) {
                CouponVo coupon = iterator.next();
                if (coupon.getCouponType() == 0 && coupon.getProductType() == 2) {
                    //活动券
                    List<Integer> productIds = pageMapper.findAllProductId(coupon.getCouponId(), source);
                    if (!productIds.contains(product.getProductId())) {
                        iterator.remove();
                        continue;
                    }
                }
                if (coupon.getCouponType() == 1 && !coupon.getCategoryType().equals(product.getCommodityType())) {
                    iterator.remove();
                    continue;
                }
                if (coupon.getCouponType() == 3 && !coupon.getShopId().equals(product.getShopId())) {
                    iterator.remove();
                    continue;
                }
                setShopName(coupon);
            }
        }
    }

    /**
     * 请求发短信接口
     *
     * @param markId  跳转类型
     * @param mobile  手机号
     * @param content 内容
     */
    public boolean getSendSms(int markId, String mobile, String content) {
        AtomicBoolean returnValue = new AtomicBoolean(false);
        try {
            //测试环境手机号允许发短信
            List<String> list = new ArrayList<>();
            list.add("15917362709");
            list.add("15814011616");
            list.add("13100721916");
            list.add("15113936829");
            list.add("18175515644");
            list.add("18476937515");
            if ("prod".equals(profile) || list.contains(mobile)) {
                if (StringUtils.isNotBlank(mobile) && mobile.length() == 11) {
                    String regex = "^(1[3-9]\\d{9}$)";
                    Pattern pattern = Pattern.compile(regex);
                    Matcher matcher = pattern.matcher(mobile);
                    if (matcher.matches()) {
                        // 调用 ToolsFeign 发送短信
                        String jsonStr = toolsFeign.getSendSms(mobile, content);
                        JSONObject parseObject = JSONObject.parseObject(jsonStr);
                        if (0 == parseObject.getInteger("code")) {
                            returnValue.set(true);
                        }
                    }
                }
            }
        } catch (Exception e) {

        }
        return returnValue.get();
    }

}
