package com.caimei365.commodity.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.opensearch.sdk.dependencies.com.google.common.collect.Lists;
import com.caimei365.commodity.components.PriceUtilService;
import com.caimei365.commodity.mapper.PriceMapper;
import com.caimei365.commodity.mapper.ShopMapper;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.vo.LadderPriceVo;
import com.caimei365.commodity.model.vo.PriceVo;
import com.caimei365.commodity.model.vo.ProductSalesVo;
import com.caimei365.commodity.model.vo.TaxVo;
import com.caimei365.commodity.service.PageService;
import com.caimei365.commodity.service.PriceService;
import com.caimei365.commodity.service.SearchIndexService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.util.List;

import static com.alibaba.fastjson.JSON.parseArray;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/9
 */
@Slf4j
@Service
public class PriceServiceImpl implements PriceService {
    @Resource
    private PriceMapper priceMapper;
    @Resource
    private ShopMapper shopMapper;
    @Resource
    private PriceUtilService priceUtilService;
    @Resource
    private PageService pageService;
    @Resource
    private SearchIndexService searchIndexService;

    /**
     * 获取商品详情价格
     *
     * @param userId    用户Id
     * @param productId 商品Id
     * @return PriceVo
     */
    @Override
    public ResponseJson<PriceVo> getDetailPrice(Integer userId, Integer productId) {
        // 数据库获取基本价格信息
        PriceVo price = priceMapper.getDetailPrice(productId);
        // 根据用户id设置详细价格
        priceUtilService.setPriceByUserId(price, userId, 0);
        log.info(">>>读取商品价格，商品ID:" + productId + ",用户ID:" + userId);
        return ResponseJson.success(price);
    }

    /**
     * 获取商品列表价格
     *
     * @param userId     用户Id
     * @param productIds 商品Id
     * @return List<PriceVo>
     */
    @Override
    public ResponseJson<List<PriceVo>> getListPrice(Integer userId, String productIds, Integer source) {
        List<Integer> productIdList = Lists.newArrayList();
        if (productIds.contains(",")) {
            String[] productArr = productIds.split(",");
            for (String id : productArr) {
                productIdList.add(Integer.parseInt(id));
            }
        } else {
            productIdList.add(Integer.parseInt(productIds));
        }
        if (productIdList.size() == 0 || null == userId) {
            return ResponseJson.error("商品Id参数错误", null);
        }
        // 数据库获取基本价格信息
        List<PriceVo> priceList = priceMapper.getListPriceByProductIds(productIdList);
        for (PriceVo price : priceList) {
            // 根据用户id设置详细价格
            priceUtilService.setPriceByUserId(price, userId, 0);
            Boolean couponsLogo = pageService.setCouponsLogo(userId, price.getProductId(), source);
            price.setCouponsLogo(couponsLogo);
        }
        log.info(">>>读取商品列表价格，商品IDs:" + productIds + ",用户ID:" + userId);
        return ResponseJson.success(priceList);
    }

    /**
     * 获取阶梯价格
     *
     * @param productId 商品Id
     * @return LadderPriceVo
     */
    @Override
    public ResponseJson<List<LadderPriceVo>> getLadderPrice(Integer productId) {
        List<LadderPriceVo> ladderPrices = priceMapper.getLadderPricesByProductId(productId);
        TaxVo tax = priceMapper.getTaxByProductId(productId);
        if (!CollectionUtils.isEmpty(ladderPrices)) {
            priceUtilService.setLadderPriceList(ladderPrices, tax);
        }
        return ResponseJson.success(ladderPrices);
    }

    /**
     * 更新商品销量
     *
     * @param productInfo [ // 商品id，数量
     *                    {"productId": 2789, "productCount": 1},
     *                    {"productId": 2790, "productCount": 2}
     *                    ]
     */
    @Override
    public ResponseJson<Void> productSaleUpdate(String productInfo) {
        JSONArray infoArr = null;
        try {
            infoArr = parseArray(productInfo);
        } catch (Exception e) {
            log.error("【更新商品销量】商品信息解析异常try-catch:", e);
            return ResponseJson.error("商品信息解析异常！", null);
        }
        if (null == infoArr) {
            return ResponseJson.error("商品数据异常！", null);
        }
        // 遍历所有商品
        for (Object productObject : infoArr) {
            JSONObject productTemp = (JSONObject) productObject;
            Integer productId = (Integer) productTemp.get("productId");
            Integer sales = (Integer) productTemp.get("productCount");
            // 获取商品销量信息
            ProductSalesVo salesVo = priceMapper.getProductSalesInfo(productId);
            if (null == salesVo || ObjectUtils.isEmpty(salesVo)) {
                return ResponseJson.error("商品数据异常！", null);
            }
            int dbCount = (null == salesVo.getProductCount()) ? 0 : salesVo.getProductCount();
            salesVo.setProductCount(dbCount + sales);
            // 获取供应商名称
            String shopName = shopMapper.getShopNameByShopId(salesVo.getShopId());
            salesVo.setShopName(shopName);
            // 分类名称
            String typeName = shopMapper.getTypeName(salesVo.getBigTypeId(), salesVo.getSmallTypeId(), salesVo.getTinyTypeId());
            if (null != salesVo.getCommodityType() && 2 == salesVo.getCommodityType()) {
                salesVo.setProperty("仪器-" + typeName);
            } else {
                salesVo.setProperty("产品-" + typeName);
            }
            if (null == salesVo.getId()) {
                // 新商品
                priceMapper.insertProductSales(salesVo);
            } else {
                // 更新销量
                priceMapper.updateProductSales(salesVo);
            }
            // 添加销量记录
            priceMapper.insertProductSalesRecord(productId, sales);
            // 更新商品索引，销量存入阿里云
            searchIndexService.updateProductIndexById(productId);
            log.info("【更新商品销量】：" + salesVo);
        }
        return ResponseJson.success(null);
    }
}
