package com.caimei365.commodity.service.impl;

import com.aliyun.opensearch.sdk.dependencies.com.google.common.collect.Lists;
import com.aliyun.opensearch.sdk.dependencies.com.google.common.collect.Maps;
import com.aliyun.opensearch.sdk.dependencies.org.json.JSONArray;
import com.aliyun.opensearch.sdk.dependencies.org.json.JSONException;
import com.aliyun.opensearch.sdk.dependencies.org.json.JSONObject;
import com.aliyun.opensearch.sdk.generated.commons.OpenSearchClientException;
import com.aliyun.opensearch.sdk.generated.commons.OpenSearchException;
import com.aliyun.opensearch.sdk.generated.search.*;
import com.caimei365.commodity.components.SearchOpenService;
import com.caimei365.commodity.mapper.PageMapper;
import com.caimei365.commodity.mapper.SearchMapper;
import com.caimei365.commodity.model.ResponseJson;
import com.caimei365.commodity.model.search.ArticleListVo;
import com.caimei365.commodity.model.search.EquipmentListVo;
import com.caimei365.commodity.model.search.ProductListVo;
import com.caimei365.commodity.model.search.ShopListVo;
import com.caimei365.commodity.model.vo.*;
import com.caimei365.commodity.service.SearchQueryService;
import com.caimei365.commodity.utils.ImageUtils;
import com.caimei365.commodity.utils.Json2PojoUtil;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.util.StringUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/4/7
 */
@Slf4j
@Service
public class SearchQueryServiceImpl implements SearchQueryService {

    private static final Integer SEARCH_MAX_NUM = 500;
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private SearchMapper searchMapper;
    @Resource
    private PageMapper pageMapper;
    @Resource
    private SearchOpenService searchOpenService;

    /**
     * 获取阿里云索引Id
     *
     * @param docType 文档类型
     * @param docId   文档ID
     * @return int
     */
    @Override
    public Integer getIdByDocId(String docType, Integer docId) {
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(searchOpenService.getAppName()));
        config.setStart(0);
        config.setHits(1);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(Lists.newArrayList("id"));
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);
        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        if (null != docId) {
            if ("product".equals(docType)) {
                // 根据商品Id获取阿里云索引Id
                searchParams.setQuery("p_id:'" + docId + "'");
            } else if ("shop".equals(docType)) {
                // 根据供应商Id获取阿里云索引Id
                searchParams.setQuery("s_id:'" + docId + "'");
            } else if ("equipment".equals(docType)) {
                // 根据项目仪器Id获取阿里云索引Id
                searchParams.setQuery("e_id:'" + docId + "'");
            } else if ("article".equals(docType)) {
                // 根据文章Id获取阿里云索引Id
                searchParams.setQuery("a_id:'" + docId + "'");
            }
        } else {
            // 创建sort对象，并设置二维排序
            Sort sorter = new Sort();
            // 设置id字段降序
            sorter.addToSortFields(new SortField("id", Order.DECREASE));
            // 若id相同则以RANK相关性算分升序
            sorter.addToSortFields(new SortField("RANK", Order.INCREASE));
            //添加Sort对象参数
            searchParams.setSort(sorter);
        }
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject("id:" + docType, searchParams);
            if (1 == result.getInt("status") && result.getJSONArray("items").length() > 0) {
                return result.getJSONArray("items").getJSONObject(0).getInt("id");
            } else {
                return -1;
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档阿里云索引Id异常：" + e);
            return -1;
        }
    }

    /**
     * 根据文档类型获取阿里云索引Id集合
     *
     * @param docType  文档类型
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return map
     */
    @Override
    public Map<Integer, Integer> getMainIdsByDocType(String docType, int pageNum, int pageSize) {
        String queryStr;
        String idName;
        Map<Integer, Integer> idMap = Maps.newLinkedHashMap();
        if ("product".equals(docType)) {
            queryStr = "p_all:'1'";
            idName = "p_id";
        } else if ("shop".equals(docType)) {
            queryStr = "s_all:'1'";
            idName = "s_id";
        } else if ("equipment".equals(docType)) {
            queryStr = "e_all:'1'";
            idName = "e_id";
        } else if ("article".equals(docType)) {
            queryStr = "a_all:'1'";
            idName = "a_id";
        } else {
            return idMap;
        }
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(searchOpenService.getAppName()));
        config.setStart(pageNum - 1);
        config.setHits(pageSize);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(Lists.newArrayList("id", idName));
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);
        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        searchParams.setQuery(queryStr);
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject("idMap:" + idName, searchParams);
            if (1 == result.getInt("status")) {
                JSONArray jsonArray = result.getJSONArray("items");
                for (int i = 0; i < jsonArray.length(); i++) {
                    idMap.put(jsonArray.getJSONObject(i).getInt(idName), jsonArray.getJSONObject(i).getInt("id"));
                }
            }
            return idMap;
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档阿里云索引Id异常：" + e);
            return idMap;
        }
    }

    /**
     * 根据文档类型获取数量
     *
     * @param docType 文档类型
     * @return int
     */
    @Override
    public Integer getRecordByDocType(String docType) {
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(searchOpenService.getAppName()));
        config.setStart(0);
        config.setHits(1);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(Lists.newArrayList("id"));
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);
        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        if ("product".equals(docType)) {
            searchParams.setQuery("p_all:'1'");
        } else if ("shop".equals(docType)) {
            searchParams.setQuery("s_all:'1'");
        } else if ("equipment".equals(docType)) {
            searchParams.setQuery("e_all:'1'");
        } else if ("article".equals(docType)) {
            searchParams.setQuery("a_all:'1'");
        } else {
            return 0;
        }
        //设置统计子句
        Aggregate agg = new Aggregate();
        agg.setGroupKey(docType);
        agg.setAggFun("count()");
        agg.setAggFilter(docType + "=1");
        //添加Aggregate对象参数
        searchParams.addToAggregates(agg);
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject("count:" + docType, searchParams);
            if (1 == result.getInt("status")) {
                return result.getJSONArray("facet").getJSONObject(0)
                        .getJSONArray("items").getJSONObject(0).getInt("count");
            } else {
                return -1;
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档阿里云索引Id异常：" + e);
            return -1;
        }
    }

    /**
     * 根据文档类型获取数量(去重用)
     *
     * @param docType 文档类型
     * @param docId   文档ID
     * @return int
     */
    @Override
    public Integer getCountByDocId(String docType, Integer docId) {
        // 定义Config对象，用于设定config子句参数，指定应用名，分页，数据返回格式等等
        Config config = new Config(Lists.newArrayList(searchOpenService.getAppName()));
        config.setStart(0);
        config.setHits(1);
        // 设置搜索结果返回应用中哪些字段
        config.setFetchFields(Lists.newArrayList("id"));
        // 设置返回格式为json格式
        config.setSearchFormat(SearchFormat.JSON);
        // 创建参数对象
        SearchParams searchParams = new SearchParams(config);
        if (null != docId) {
            if ("product".equals(docType)) {
                searchParams.setQuery("p_id:'" + docId + "'");
            } else if ("shop".equals(docType)) {
                searchParams.setQuery("s_id:'" + docId + "'");
            } else if ("equipment".equals(docType)) {
                searchParams.setQuery("e_id:'" + docId + "'");
            } else if ("article".equals(docType)) {
                searchParams.setQuery("a_id:'" + docId + "'");
            } else {
                return 0;
            }
        } else {
            // 创建sort对象，并设置二维排序
            Sort sorter = new Sort();
            // 设置id字段降序
            sorter.addToSortFields(new SortField("id", Order.DECREASE));
            // 若id相同则以RANK相关性算分升序
            sorter.addToSortFields(new SortField("RANK", Order.INCREASE));
            //添加Sort对象参数
            searchParams.setSort(sorter);
        }
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject("count:" + docType, searchParams);
            if (1 == result.getInt("status") && result.getInt("total") >= 0) {
                return result.getInt("total");
            } else {
                return -1;
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档索引数量异常：" + e);
            return -1;
        }
    }

    /**
     * 根据关键词搜索供应商
     *
     * @param keyword  搜索关键字
     * @param identity 用户身份 1:协销， 2:会员机构
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> querySupplierByKeyword(String keyword, Integer identity, int pageNum, int pageSize) {
        String queryStr = "supplier:'" + keyword + "'";
        if (StringUtils.isEmpty(keyword)) {
            queryStr = "s_all:'1'";
        }
        // 阿里云搜索
        ResponseJson<String> result = querySupplier(queryStr, identity, pageNum, pageSize);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            return querySupplierFromDatabase(pageNum, pageSize, identity, keyword);
        }
    }

    /**
     * 根据关键词搜索项目仪器
     *
     * @param keyword  搜索关键字
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryEquipmentByKeyword(String keyword, int pageNum, int pageSize) {
        String queryStr = "equipment:'" + keyword + "'";
        if (StringUtils.isEmpty(keyword)) {
            queryStr = "e_all:'1'";
        }
        // 阿里云搜索
        ResponseJson<String> result = queryEquipment(queryStr, pageNum, pageSize);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            return queryEquipmentFromDatabase(pageNum, pageSize, keyword);
        }
    }

    /**
     * 根据关键词搜索文章
     *
     * @param keyword  搜索关键字
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryArticleByKeyword(String keyword, int pageNum, int pageSize, Integer status) {
        String queryStr = "article:'" + keyword + "'";
        if (StringUtils.isEmpty(keyword)) {
            queryStr = "a_all:'1'";
        }
        ArrayList<String> fetchFields = Lists.newArrayList("id", "a_id", "a_title", "a_image", "a_publisher", "a_publish_date", "a_intro", "a_pv", "a_type_id", "a_type_text", "a_label_ids", "a_label_text");
        List<Map<String, String>> summaryList = new ArrayList<>(2);
        Map<String, String> summaryField = new HashMap<>(2);
        summaryField.put("length", "100");
        summaryField.put("field", "a_title");
        summaryList.add(summaryField);
        Map<String, String> summaryField2 = new HashMap<>(2);
        summaryField2.put("length", "200");
        summaryField2.put("field", "a_intro");
        summaryList.add(summaryField2);
        // 阿里云搜索
        String sort = 2 == status ? "a_publish_date" : null;
        ResponseJson<String> result = queryArticleForParams(queryStr, fetchFields, summaryList, pageNum, pageSize, sort, 1);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            //1.2.8版本新加根据时间排序
            if (status == 1) {
                //综合排序
                return queryArticleFromDatabase(pageNum, pageSize, keyword, null, null, null);
            } else {
                //根据时间排序
                return queryArticleFromDatabaseByProDate(pageNum, pageSize, keyword, null, null, null);
            }

        }
    }

    /**
     * 根据文章ID搜索文章
     *
     * @param id 文章id
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryArticleByArticleId(Integer id) {
        String queryStr = "a_id:'" + id + "'";
        ArrayList<String> fetchFields = Lists.newArrayList("id", "a_id", "a_title", "a_image", "a_publisher", "a_publish_date", "a_intro", "a_pv", "a_type_id", "a_type_text", "a_label_ids", "a_label_text");
        // 阿里云搜索
        ResponseJson<String> result = queryArticleForParams(queryStr, fetchFields, null, 1, 1, null, 1);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            return queryArticleFromDatabase(1, 1, null, id, null, null);
        }
    }

    /**
     * 根据分类搜索文章
     *
     * @param id       分类id
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryArticleByType(Integer id, int pageNum, int pageSize) {
        String queryStr = "a_type:'" + id + "'";
        ArrayList<String> fetchFields = Lists.newArrayList("id", "a_id", "a_title", "a_image", "a_publisher", "a_publish_date", "a_intro", "a_pv", "a_type_id", "a_type_text", "a_label_ids", "a_label_text");
        // 阿里云搜索
        ResponseJson<String> result = scrollArticleForParams(queryStr, fetchFields, pageNum, pageSize, "a_id", 1);
        if (0 == result.getCode()) {
            return result;
        } else {
            // 阿里云搜索失败，再次从数据库搜索
            return queryArticleFromDatabase(pageNum, pageSize, null, null, id, null);
        }
    }

    /**
     * 根据标签搜索文章
     *
     * @param id       标签id
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return JsonStr(list)
     */
    @Override
    public ResponseJson<String> queryArticleByLabel(Integer id, int pageNum, int pageSize) {
//        String queryStr = "a_label:'" + id + "'";
//        ArrayList<String> fetchFields = Lists.newArrayList("id", "a_id", "a_title", "a_image", "a_publisher", "a_publish_date", "a_intro", "a_pv", "a_type_id", "a_type_text", "a_label_ids", "a_label_text");
//        // 阿里云搜索
//        ResponseJson<String> result = scrollArticleForParams(queryStr, fetchFields, pageNum, pageSize, "a_id", 1);
//        if (0 == result.getCode()) {
//            return result;
//        } else {
        // 阿里云搜索失败，再次从数据库搜索
        return queryArticleFromDatabase(pageNum, pageSize, null, null, null, id);
//        }
    }

    /**
     * 文章查询(数据库)
     *
     * @param keyword   关键词
     * @param articleId 文章Id
     * @param typeId    分类Id
     * @param labelId   标签Id
     * @param num       页码
     * @param size      每页数量
     */
    private ResponseJson<String> queryArticleFromDatabase(int num, int size, String keyword, Integer articleId, Integer typeId, Integer labelId) {
        String labelText = null;
        if (null != labelId) {
            labelText = searchMapper.findLabelNameById(labelId);
        }
        PageHelper.startPage(num, size);
        List<ArticleListVo> articleList = searchMapper.queryArticle(keyword, articleId, typeId, labelText);
        articleList.forEach(article -> {
            String imagePath = ImageUtils.getImageURL("", article.getImage(), 0, domain);
            article.setImage(imagePath);
            // 搜索供应商文档数据
            String[] labelTexts = article.getLabelText().split(",");
            if (labelTexts.length > 0) {
                boolean notEmpty = StringUtil.isNotEmpty(labelTexts[0]);
                if (notEmpty) {
                    List<Integer> labelIdList = searchMapper.findLabelIdsByName(labelTexts);
                    StringBuilder labelIds = new StringBuilder();
                    labelIdList.forEach(id -> {
                        labelIds.append(",").append(id);
                    });
                    article.setLabelIds(labelIds.toString().replaceFirst(",", ""));
                }
            }
        });
        PaginationVo<ArticleListVo> pageData = new PaginationVo<>(articleList);
        int total = pageData.getTotalRecord();
        JSONObject pageObj = new JSONObject();
        pageObj.put("total", total);
        pageObj.put("items", articleList);
        log.info(">>>>>>数据库容错查询: pageSize(" + size + "),pageNum(" + size + "),total(" + total + ")");
        if (articleList.size() > 0) {
            return ResponseJson.success(pageObj.toString());
        } else {
            return ResponseJson.error("未查询到文档记录(数据库)", null);
        }
    }

    /**
     * 1.2.8新加发布时间排序
     * 文章查询(数据库)
     *
     * @param keyword   关键词
     * @param articleId 文章Id
     * @param typeId    分类Id
     * @param labelId   标签Id
     * @param num       页码
     * @param size      每页数量
     */
    private ResponseJson<String> queryArticleFromDatabaseByProDate(int num, int size, String keyword, Integer articleId, Integer typeId, Integer labelId) {
        String labelText = null;
        if (null != labelId) {
            labelText = searchMapper.findLabelNameById(labelId);
        }
        PageHelper.startPage(num, size);
        List<ArticleListVo> articleList = searchMapper.queryArticleByProDate(keyword, articleId, typeId, labelText);
        articleList.forEach(article -> {
            String imagePath = ImageUtils.getImageURL("", article.getImage(), 0, domain);
            article.setImage(imagePath);
            // 搜索供应商文档数据
            String[] labelTexts = article.getLabelText().split(",");
            if (labelTexts.length > 0) {
                boolean notEmpty = StringUtil.isNotEmpty(labelTexts[0]);
                if (notEmpty) {
                    List<Integer> labelIdList = searchMapper.findLabelIdsByName(labelTexts);
                    StringBuilder labelIds = new StringBuilder();
                    labelIdList.forEach(id -> {
                        labelIds.append(",").append(id);
                    });
                    article.setLabelIds(labelIds.toString().replaceFirst(",", ""));
                }
            }
        });
        PaginationVo<ArticleListVo> pageData = new PaginationVo<>(articleList);
        int total = pageData.getTotalRecord();
        JSONObject pageObj = new JSONObject();
        pageObj.put("total", total);
        pageObj.put("items", articleList);
        log.info(">>>>>>数据库容错查询: pageSize(" + size + "),pageNum(" + size + "),total(" + total + ")");
        if (articleList.size() > 0) {
            return ResponseJson.success(pageObj.toString());
        } else {
            return ResponseJson.error("未查询到文档记录(数据库)", null);
        }
    }

    /**
     * 文章查询
     *
     * @param queryStr      查询类型及参数
     * @param fetchFields   返回字段
     * @param summaryFields 摘要字段
     * @param num           页码
     * @param size          每页数量
     */
    private ResponseJson<String> queryArticleForParams(String queryStr, ArrayList<String> fetchFields, List<Map<String, String>> summaryFields, int num, int size, String sortField, Integer sortType) {
        SearchParams searchParams;
        int requestSize = num * size;
        // 由于阿里云搜索机制问题（分页数据重复），搜索500条以内数据手动分页。
        if (requestSize > SEARCH_MAX_NUM) {
            searchParams = searchOpenService.getSearchParams(queryStr, fetchFields, summaryFields, "article", num, size, sortField, sortType);
        } else {
            searchParams = searchOpenService.getSearchParams(queryStr, fetchFields, summaryFields, "article", 1, requestSize, sortField, sortType);
        }
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject(queryStr, searchParams);
            if (1 == result.getInt("status")) {
                int total = result.getInt("total");
                int totalPage = (int) Math.ceil((double) total / size);
                JSONArray resultArr = result.getJSONArray("items");
                JSONArray pageData = new JSONArray();
                if (requestSize > SEARCH_MAX_NUM) {
                    pageData = resultArr;
                } else {
                    if (resultArr.length() > 0) {
                        int baseNum = (num - 1) * size;
                        for (int i = 0; i < size; i++) {
                            int index = baseNum + i;
                            int currentTotal = requestSize;
                            if (totalPage == num) {
                                currentTotal = total;
                            }
                            if (index < currentTotal) {
                                JSONObject item = resultArr.getJSONObject(index);
                                pageData.put(i, item);
                            } else {
                                break;
                            }
                        }
                    }
                }
                // 阿里云查询出json转换成文章pojo
                List<ArticleListVo> articleList = Json2PojoUtil.toArticleList(pageData);
                JSONObject pageObj = new JSONObject();
                pageObj.put("total", total);
                pageObj.put("items", articleList);
                log.info(">>>>>>pageResult(" + queryStr + "): pageSize(" + size + "),pageNum(" + num + "),total(" + total + ")");
                return ResponseJson.success(pageObj.toString());
            } else {
                return ResponseJson.error("未查询到文档记录", null);
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档异常：" + e.getMessage());
            return ResponseJson.error("查询文档异常：" + e.getMessage(), null);
        }
    }

    /**
     * 滚动查询文章数据
     *
     * @param queryStr  查询类型及参数
     * @param pageNum   页码
     * @param pageSize  每页数量
     * @param sortField 排序字段
     * @param sortType  升降序
     */
    private ResponseJson<String> scrollArticleForParams(String queryStr, ArrayList<String> fetchFields, int pageNum, int pageSize, String sortField, Integer sortType) {
        int requestSize = pageNum * pageSize;
        int requestPageSize = requestSize;
        if (requestSize > SEARCH_MAX_NUM) {
            requestPageSize = (int) Math.floor(SEARCH_MAX_NUM / pageSize) * pageSize;
        }
        SearchParams searchParams = searchOpenService.getScrollParams(queryStr, fetchFields, requestPageSize, sortField, sortType);
        ResponseJson<String> result;
        try {
            JSONObject pageObj = searchOpenService.deepPagingForParams(searchParams, queryStr, SEARCH_MAX_NUM, requestPageSize, pageNum, pageSize);
            if (1 == pageObj.getInt("status")) {
                JSONArray pageData = pageObj.getJSONArray("items");
                // 阿里云查询出json转换成商品pojo
                List<ArticleListVo> articleList = Json2PojoUtil.toArticleList(pageData);
                pageObj.put("items", articleList);
                result = ResponseJson.success(pageObj.toString());
            } else {
                result = ResponseJson.error("未查询到文档记录", null);
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档异常：" + e.getMessage());
            result = ResponseJson.error("查询文档异常：" + e.getMessage(), null);
        }
        return result;
    }

    /**
     * 获取项目仪器(数据库)
     *
     * @param num     页码
     * @param size    每页数量
     * @param keyword 关键词
     */
    private ResponseJson<String> queryEquipmentFromDatabase(int num, int size, String keyword) {
        PageHelper.startPage(num, size);
        List<EquipmentListVo> equipmentList = searchMapper.queryEquipment(keyword);
        PaginationVo<EquipmentListVo> pageData = new PaginationVo<>(equipmentList);
        int total = pageData.getTotalRecord();
        JSONObject pageObj = new JSONObject();
        pageObj.put("total", total);
        pageObj.put("items", equipmentList);
        log.info(">>>>>>数据库容错查询: pageSize(" + size + "),pageNum(" + size + "),total(" + total + ")");
        if (equipmentList.size() > 0) {
            return ResponseJson.success(pageObj.toString());
        } else {
            return ResponseJson.error("未查询到文档记录(数据库)", null);
        }
    }

    /**
     * 查询项目仪器（阿里云搜索）
     *
     * @param queryStr 查询类型及参数
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    private ResponseJson<String> queryEquipment(String queryStr, int pageNum, int pageSize) {
        ArrayList<String> fetchFields = Lists.newArrayList("id", "e_id", "e_name", "e_image");
        List<Map<String, String>> summaryList = new ArrayList<>(1);
        Map<String, String> summaryField = new HashMap<>(2);
        summaryField.put("length", "100");
        summaryField.put("field", "e_name");
        summaryList.add(summaryField);
        // 由于阿里云搜索机制问题（分页数据重复），搜索500条以内数据手动分页。
        SearchParams searchParams;
        int requestSize = pageNum * pageSize;
        if (requestSize > SEARCH_MAX_NUM) {
            searchParams = searchOpenService.getSearchParams(queryStr, fetchFields, summaryList, "equipment", pageNum, pageSize, null, null);
        } else {
            searchParams = searchOpenService.getSearchParams(queryStr, fetchFields, summaryList, "equipment", 1, requestSize, null, null);
        }
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject(queryStr, searchParams);
            if (1 == result.getInt("status")) {
                JSONArray resultArr = result.getJSONArray("items");
                JSONArray pageData = new JSONArray();
                int total = result.getInt("total");
                int totalPage = (int) Math.ceil((double) total / pageSize);
                if (requestSize > SEARCH_MAX_NUM) {
                    pageData = resultArr;
                } else {
                    if (resultArr.length() > 0) {
                        int baseNum = (pageNum - 1) * pageSize;
                        for (int i = 0; i < pageSize; i++) {
                            int index = baseNum + i;
                            int currentTotal = requestSize;
                            if (totalPage == pageNum) {
                                currentTotal = total;
                            }
                            if (index < currentTotal) {
                                JSONObject item = resultArr.getJSONObject(index);
                                pageData.put(i, item);
                            } else {
                                break;
                            }
                        }
                    }
                }
                // 阿里云查询出json转换项目仪器pojo
                List<EquipmentListVo> equipmentList = Json2PojoUtil.toEquipmentList(pageData);
                JSONObject pageObj = new JSONObject();
                pageObj.put("total", total);
                pageObj.put("items", equipmentList);
                log.info(">>>>>>pageResult(" + queryStr + "): pageSize(" + pageSize + "),pageNum(" + pageNum + "),total(" + total + ")");
                return ResponseJson.success(pageObj.toString());
            } else {
                return ResponseJson.error("未查询到文档记录", null);
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档异常：" + e.getMessage());
            return ResponseJson.error("查询文档异常：" + e.getMessage(), null);
        }
    }

    /**
     * 获取供应商(数据库)
     *
     * @param num      页码
     * @param size     每页数量
     * @param identity 用户身份
     * @param keyword  关键词
     */
    private ResponseJson<String> querySupplierFromDatabase(int num, int size, Integer identity, String keyword) {
        PageHelper.startPage(num, size);
        List<ShopListVo> shopList = searchMapper.querySupplier(keyword);
        shopList.forEach(shop -> {
            PageHelper.startPage(1, 3);
            List<ProductListVo> productList = searchMapper.queryProduct(identity, null, shop.getShopId(), null, null, null, null, null, null);
            shop.setProducts(productList);
        });
        PaginationVo<ShopListVo> pageData = new PaginationVo<>(shopList);
        int total = pageData.getTotalRecord();
        JSONObject pageObj = new JSONObject();
        pageObj.put("total", total);
        pageObj.put("items", shopList);
        log.info(">>>>>>数据库容错查询: pageSize(" + size + "),pageNum(" + size + "),total(" + total + ")");
        if (shopList.size() > 0) {
            return ResponseJson.success(pageObj.toString());
        } else {
            return ResponseJson.error("未查询到文档记录(数据库)", null);
        }
    }

    /**
     * 查询供应商（阿里云搜索）
     *
     * @param queryStr 查询类型及参数
     * @param identity 用户身份 1:协销， 2:会员机构
     * @param pageNum  页码
     * @param pageSize 每页数量
     */
    private ResponseJson<String> querySupplier(String queryStr, Integer identity, int pageNum, int pageSize) {
        ArrayList<String> fetchFields = Lists.newArrayList("id", "s_id", "s_name", "s_logo", "s_business", "s_address", "s_license");
        // 由于阿里云搜索机制问题（分页数据重复），搜索500条以内数据手动分页。
        SearchParams searchParams;
        int requestSize = pageNum * pageSize;
        if (requestSize > SEARCH_MAX_NUM) {
            searchParams = searchOpenService.getSearchParams(queryStr, fetchFields, null, "supplier", pageNum, pageSize, "s_sort", 1);
        } else {
            searchParams = searchOpenService.getSearchParams(queryStr, fetchFields, null, "supplier", 1, requestSize, "s_sort", 1);
        }
        try {
            // 查询推送
            JSONObject result = searchOpenService.getResultJsonObject(queryStr, searchParams);
            if (1 == result.getInt("status")) {
                int total = result.getInt("total");
                int totalPage = (int) Math.ceil((double) total / pageSize);
                JSONArray resultArr = result.getJSONArray("items");
                JSONArray pageData = new JSONArray();
                if (requestSize > SEARCH_MAX_NUM) {
                    pageData = resultArr;
                } else {
                    if (resultArr.length() > 0) {
                        int baseNum = (pageNum - 1) * pageSize;
                        for (int i = 0; i < pageSize; i++) {
                            int index = baseNum + i;
                            int currentTotal = requestSize;
                            if (totalPage == pageNum) {
                                currentTotal = total;
                            }
                            if (index < currentTotal) {
                                JSONObject item = resultArr.getJSONObject(index);
                                pageData.put(i, item);
                            } else {
                                break;
                            }
                        }
                    }
                }
                for (int i = 0; i < pageData.length(); i++) {
                    JSONObject supplier = pageData.getJSONObject(i);
                    int shopId = supplier.getInt("s_id");
                    String productQuery = "p_supplier:'" + shopId + "'";
                    SearchParams productSearchParams = searchOpenService.getProductParams(productQuery, "", identity, 1, 3, "", null);
                    // 查询商品
                    JSONObject pResult = searchOpenService.getResultJsonObject(productQuery, productSearchParams);
                    if (1 == pResult.getInt("status")) {
                        JSONArray productArr = pResult.getJSONArray("items");
                        supplier.put("products", productArr);
                    }
                }
                // 阿里云查询出json转换成供应商pojo
                List<ShopListVo> shopList = Json2PojoUtil.toShopList(pageData);
                JSONObject pageObj = new JSONObject();
                pageObj.put("total", total);
                pageObj.put("items", shopList);
                log.info(">>>>>>pageResult(" + queryStr + "): pageSize(" + pageSize + "),pageNum(" + pageNum + "),total(" + total + ")");
                return ResponseJson.success(pageObj.toString());
            } else {
                return ResponseJson.error("未查询到文档记录", null);
            }
        } catch (OpenSearchClientException | OpenSearchException | JSONException e) {
            log.error("查询文档异常：" + e.getMessage());
            return ResponseJson.error("查询文档异常：" + e.getMessage(), null);
        }
    }

    @Override
    public ResponseJson<PaginationVo<BaikeProductVo>> queryBaikeProductByKeyword(String keyword, int pageNum, int pageSize) {
        if (StringUtils.isEmpty(keyword)) {
            return ResponseJson.error("参数异常", null);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<BaikeProductVo> baikeProducts = pageMapper.getBaikeProducts(null, keyword, null);
        baikeProducts.forEach(product -> {
            // 问题列表
            List<String> questionList = pageMapper.getBaikeQuestionList(product.getProductId());
            product.setQuestionList(questionList);
        });
        PaginationVo<BaikeProductVo> pageData = new PaginationVo<>(baikeProducts);
        return ResponseJson.success(pageData);
    }
}
