package com.caimei365.commodity.utils;

import org.apache.commons.lang.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/5
 */
public class ValidateUtil {
    /** 手机号码验证格式 */
    private static final String MOBILE_PATTERN = "^1[1-9]\\d{9}$";
    /** 邮箱验证格式 */
    private static final String EMAIL_PATTERN = "^\\s*?(.+)@(.+?)\\s*$";
    /** 密码验证格式 */
    private static final String PASSWORD_PATTERN = "^(?![0-9]+$)(?![a-zA-Z]+$)[0-9A-Za-z]{8,16}$";
    /** 机构名称验证格式 */
    private static final String CLUB_NAME_PATTERN = "^[a-zA-Z0-9\u4e00-\u9fa5]{1,200}$";
    /** 统一社会信用代码验证格式 */
    private static final String SOCIAL_CREDIT_CODE_PATTERN = "^[0-9A-HJ-NPQRTUWXY]{2}\\d{6}[0-9A-HJ-NPQRTUWXY]{10}$";

    /**
     * 验证手机号码格式
     *
     * @param mobile 手机号
     */
    public static String validateMobile(String mobile) {
        if (StringUtils.isEmpty(mobile) || StringUtils.isBlank(mobile)) {
            return "手机号不可为空";
        } else {
            Pattern pattern = Pattern.compile(MOBILE_PATTERN);
            Matcher matcher = pattern.matcher(mobile);
            if (matcher.matches()) {
                return null;
            } else {
                return "手机号码格式不正确";
            }
        }
    }

    /**
     * 验证简单的邮箱格式
     *
     * @param email 邮箱
     */
    public static boolean validateEmail(String email) {
        if (StringUtils.isEmpty(email) || StringUtils.isBlank(email)) {
            return false;
        } else {
            Pattern pattern = Pattern.compile(EMAIL_PATTERN);
            Matcher matcher = pattern.matcher(email);
            return matcher.matches();
        }
    }

    /**
     * 密码验证
     */
    public static String validatePassWord(String password) {
        if (StringUtils.isEmpty(password) || StringUtils.isBlank(password)) {
            return "密码不可为空";
        }
        if (password.length() > 16 || password.length() < 8) {
            return "密码长度不正确(8-16)";
        }
        Pattern pattern = Pattern.compile(PASSWORD_PATTERN);
        Matcher matcher = pattern.matcher(password);
        if (matcher.matches()) {
            return null;
        } else {
            return "请输入8-16位字符，需字母数字组合。";
        }
    }

    /**
     * 验证机构名称
     */
    public static String validateClubName(String name) {
        if (StringUtils.isEmpty(name) || StringUtils.isBlank(name)) {
            return "请填写美容机构名称";
        } else {
            Pattern pattern = Pattern.compile(CLUB_NAME_PATTERN);
            Matcher matcher = pattern.matcher(name);
            if (matcher.matches()) {
                return null;
            } else {
                return "机构名称格式不正确";
            }
        }
    }

    /**
     * 统一社会信用代码验证
     */
    public static String validateSocialCreditCode(String socialCreditCode) {
        if (StringUtils.isEmpty(socialCreditCode) || StringUtils.isBlank(socialCreditCode)) {
            return "请输入您的统一社会信用代码";
        } else {
            Pattern pattern = Pattern.compile(SOCIAL_CREDIT_CODE_PATTERN);
            Matcher matcher = pattern.matcher(socialCreditCode);
            if (matcher.matches()) {
                return null;
            } else {
                return "统一社会信用代码格式不正确";
            }
        }
    }

    /**
     * 名称（或联系人姓名）1个字的话，全部显示
     * 名称（或联系人姓名）2个字的话，显示第二个字，第一个字打星号*
     * 名称（或联系人姓名）3个字的话，显示后两个字，第一个字打星号*
     * 名称（或联系人姓名）4个字话，显示前两个字和后一个字，中间的用一个星号*表示
     * 名称（或联系人姓名）5个字及以上的话，显示前两个字和后一个字，中间的统一用3个星号*表示
     *
     * @param  name
     * @return String
     */
    public static String nameConversion(String name){
        char[] nameArr = name.toCharArray();
        StringBuilder showName = new StringBuilder();
        switch (nameArr.length) {
            case 0:
            case 1:
                break;
            case 2:
                showName.append('*').append(nameArr[1]);
                break;
            case 3:
                showName.append('*').append(nameArr[1]).append(nameArr[2]);
                break;
            case 4:
                showName.append(nameArr[0]).append(nameArr[1]).append('*').append(nameArr[3]);
                break;
            default:
                showName.append(nameArr[0]).append(nameArr[1]).append("***").append(nameArr[nameArr.length-1]);
        }
        return showName.toString();
    }

}
